/* === This file is part of Calamares - <https://github.com/calamares> ===
 *
 *   Copyright 2014-2017, Teo Mrnjavac <teo@kde.org>
 *   Copyright 2017, Adriaan de Groot <groot@kde.org>
 *
 *   Calamares is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Calamares is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Calamares. If not, see <http://www.gnu.org/licenses/>.
 */

#include <MarinerConfJob.h>

#include "JobQueue.h"
#include "GlobalStorage.h"
#include "utils/Logger.h"
#include "utils/CalamaresUtilsSystem.h"

#include <QString>
#include <QJsonObject>
#include <QJsonArray>
#include <QJsonDocument>
#include <QFile>
#include <QFileInfo>

#include <random>
#include <sstream>

#ifndef NO_CRYPT_H
#include <crypt.h>
#endif
#include <unistd.h>

#include <fstream>

using std::string;

MarinerconfJob::MarinerconfJob( const QString& hostname,
                                const QString& userName,
                                const QString& userPassword,
                                const string& ksPath )
    : Calamares::Job()
    , m_userPassword( userPassword )
    , m_userName( userName )
    , m_hostname( hostname )
    , m_ksPath( ksPath )
{
}


QString
MarinerconfJob::prettyName() const
{
    return tr( "Set configuration for root" );
}


QString
MarinerconfJob::prettyStatusMessage() const
{
    return tr( "Setting CBL-Mariner configuration for root." );
}


Calamares::JobResult
MarinerconfJob::exec()
{
    const QString KeyHostname = "Hostname";
    const QString KeyUsers = "Users";
    const QString KeyName = "Name";
    const QString KeyPassword = "Password";
    const QString KeySecondaryGroups = "SecondaryGroups";
    const QString KeyPartitionSettings = "PartitionSettings";
    const QString KeySystemConfigs = "SystemConfigs";
    const QString MarinerConfGS = "MARINERCONF";
    const QString GroupWheel = "wheel";

    Calamares::GlobalStorage* gs = Calamares::JobQueue::instance()->globalStorage();
    QJsonObject marinerConfiguration = gs->value(MarinerConfGS).toJsonObject();
    QJsonObject systemConfiguration = marinerConfiguration[KeySystemConfigs].toArray()[0].toObject();

    // Passwordless root
    QJsonObject usersRoot;
    usersRoot[KeyName] = "root";

    QJsonObject usersUser;
    usersUser[KeyName] = m_userName;
    usersUser[KeyPassword] = m_userPassword;
    // User is part of sudoers group - wheel
    usersUser[KeySecondaryGroups] = QJsonArray() << GroupWheel;

    systemConfiguration[KeyHostname] = m_hostname;
    systemConfiguration[KeyUsers] = QJsonArray() << usersRoot << usersUser;
    marinerConfiguration[KeySystemConfigs] = QJsonArray() << systemConfiguration;
    QFile marinerFile(QString::fromStdString(m_ksPath));
    if(!marinerFile.open(QFile::WriteOnly))
    {
        QFileInfo fi(marinerFile);
        return Calamares::JobResult::error( tr( "Bad destination system config path." ),
                                            tr( "file is %1" ).arg( fi.filePath() ) );
    }

    QJsonDocument configurationJson(marinerConfiguration);
    marinerFile.write(configurationJson.toJson());
    return Calamares::JobResult::ok();
}
